@@ -1,7 +1,7 @@
 /*
     SPDX-License-Identifier: GPL-2.0-or-later
     SPDX-FileCopyrightText: 2009-2012 Alexander Rieder <alexanderrieder@gmail.com>
-    SPDX-FileCopyrightText: 2017-2021 by Alexander Semke (alexander.semke@web.de)
+    SPDX-FileCopyrightText: 2017-2022 by Alexander Semke (alexander.semke@web.de)
 */
 
 #include "maximaexpression.h"
@@ -38,7 +38,6 @@ MaximaExpression::~MaximaExpression() {
 
 void MaximaExpression::evaluate()
 {
-    m_isHelpRequest = false;
     m_gotErrorContent = false;
 
     if(m_tempFile)
@@ -66,18 +65,17 @@ void MaximaExpression::evaluate()
         || cmd.startsWith(QLatin1String("describe("))
         || cmd.startsWith(QLatin1String("example("))
         || cmd.startsWith(QLatin1String(":lisp(cl-info::info-exact")))
-        m_isHelpRequest=true;
+        setIsHelpRequest(true);
 
     if (MaximaSettings::self()->integratePlots()
         && !cmd.contains(QLatin1String("ps_file"))
-        && cmd.contains(QRegularExpression(QStringLiteral("(?:plot2d|plot3d|contour_plot)\\s*\\([^\\)]"))))
+        && cmd.contains(QRegularExpression(QStringLiteral("(?:plot2d|plot3d|contour_plot|draw|draw2d|draw3d)\\s*\\([^\\)]"))))
     {
-        m_isPlot=true;
-#ifdef WITH_EPS
-        m_tempFile=new QTemporaryFile(QDir::tempPath() + QLatin1String("/cantor_maxima-XXXXXX.eps" ));
-#else
-        m_tempFile=new QTemporaryFile(QDir::tempPath() + QLatin1String("/cantor_maxima-XXXXXX.png"));
-#endif
+        m_isPlot = true;
+        if (cmd.contains(QRegularExpression(QStringLiteral("(?:draw|draw2d|draw3d)\\s*\\([^\\)]"))))
+            m_isDraw = true;
+
+        m_tempFile = new QTemporaryFile(QDir::tempPath() + QLatin1String("/cantor_maxima-XXXXXX.png"));
         m_tempFile->open();
 
         m_fileWatch.removePaths(m_fileWatch.files());
@@ -152,17 +150,28 @@ QString MaximaExpression::internalCommand()
             qDebug()<<"plotting without tempFile";
             return QString();
         }
-        QString fileName = m_tempFile->fileName();
 
-#ifdef WITH_EPS
-        const QString psParam=QLatin1String("[gnuplot_ps_term_command, \"set size 1.0,  1.0; set term postscript eps color solid \"]");
-        const QString plotParameters = QLatin1String("[ps_file, \"")+ fileName+QLatin1String("\"],")+psParam;
-#else
-        const QString plotParameters = QLatin1String("[gnuplot_term, \"png size 500,340\"], [gnuplot_out_file, \"")+fileName+QLatin1String("\"]");
 
-#endif
-        cmd.replace(QRegularExpression(QStringLiteral("((plot2d|plot3d|contour_plot)\\s*\\(.*)\\)([;\n$]|$)")),
-                    QLatin1String("\\1, ") + plotParameters + QLatin1String(");"));
+        /*
+        const QString pdfParam = QLatin1String("[gnuplot_pdf_term_command, \"set term pdfcairo size 9cm, 6cm\"]"); // size 5.9in, 4.7in
+        const QString plotParameters = QLatin1String("[pdf_file, \"") + fileName + QLatin1String("\"], ") + pdfParam;
+        */
+
+        if (!m_isDraw)
+        {
+            const QString plotParameters = QLatin1String("[gnuplot_png_term_command, \"set term png size 500,340\"], [png_file, \"") + m_tempFile->fileName() + QLatin1String("\"]");
+            cmd.replace(QRegularExpression(QStringLiteral("((plot2d|plot3d|contour_plot)\\s*\\(.*)\\)([;\n$]|$)")),
+                        QLatin1String("\\1, ") + plotParameters + QLatin1String(");"));
+        }
+        else
+        {
+            //strip off the extension ".png", the terminal parameter for draw() is adding the extension additionally
+            QString fileName = m_tempFile->fileName();
+            fileName = fileName.left(fileName.length() - 4);
+            const QString plotParameters = QLatin1String("terminal=png, file_name = \"") + fileName + QLatin1String("\"");
+            cmd.replace(QRegularExpression(QStringLiteral("((draw|draw2d|draw3d)\\s*\\(.*)*\\)([;\n$]|$)")),
+                        QLatin1String("\\1, ") + plotParameters + QLatin1String(");"));
+        }
 
     }
 
@@ -198,7 +207,7 @@ void MaximaExpression::forceDone()
     latex mode - "<cantor-result><cantor-text>\n(%o1) 10\n</cantor-text><cantor-latex>\\mbox{\\tt\\red(\\mathrm{\\%o1}) \\black}10</cantor-latex></cantor-result>\n<cantor-prompt>(%i2) </cantor-prompt>\n"
     text mode  - "<cantor-result><cantor-text>\n(%o1) 10\n</cantor-text></cantor-result>\n<cantor-prompt>(%i2) </cantor-prompt>\n"
  */
-bool MaximaExpression::parseOutput(QString& out)
+void MaximaExpression::parseOutput(const QString& out)
 {
     const int promptStart = out.indexOf(QLatin1String("<cantor-prompt>"));
     const int promptEnd = out.indexOf(QLatin1String("</cantor-prompt>"));
@@ -214,7 +223,7 @@ bool MaximaExpression::parseOutput(QString& out)
 
         qDebug()<<"asking for additional input for " << textContent;
         emit needsAdditionalInformation(textContent);
-        return true;
+        return;
     }
 
     qDebug()<<"new input label: " << prompt;
@@ -223,8 +232,23 @@ bool MaximaExpression::parseOutput(QString& out)
 
     //parse the results
     int resultStart = out.indexOf(QLatin1String("<cantor-result>"));
-    if (resultStart != -1)
+    if (resultStart != -1) {
         errorContent += out.mid(0, resultStart);
+        if (!errorContent.isEmpty() && !(isHelpRequest() || m_isHelpRequestAdditional))
+        {
+            //there is a result but also the error buffer is not empty. This is the case when
+            //warnings are generated, for example, the output of rat(0.75*10) is:
+            //"\nrat: replaced 7.5 by 15/2 = 7.5\n<cantor-result><cantor-text>\n(%o2) 15/2\n</cantor-text></cantor-result>\n<cantor-prompt>(%i3) </cantor-prompt>\n".
+            //In such cases we just add a new text result with the warning.
+            qDebug() << "warning: " << errorContent;
+            auto* result = new Cantor::TextResult(errorContent.trimmed());
+
+            //the output of tex() function is also placed outside of the result section, don't treat it as a warning
+            if (!command().remove(QLatin1Char(' ')).startsWith(QLatin1String("tex(")))
+                result->setIsWarning(true);
+            addResult(result);
+        }
+    }
 
     while (resultStart != -1)
     {
@@ -254,14 +278,39 @@ bool MaximaExpression::parseOutput(QString& out)
     {
         qDebug() << "error content: " << errorContent;
 
-        if (out.contains(QLatin1String("cantor-value-separator")))
+        if (out.contains(QLatin1String("cantor-value-separator"))
+            || (out.contains(QLatin1String("<cantor-result>")) && !(isHelpRequest() || m_isHelpRequestAdditional)) )
         {
-            //when fetching variables, in addition to the actual result with variable names and values,
-            //Maxima also writes out the names of the variables to the error buffer.
-            //we don't interpret this as an error.
+            //we don't interpret the error output as an error in the following cases:
+            //1. when fetching variables, in addition to the actual result with variable names and values,
+            //  Maxima also writes out the names of the variables to the error buffer.
+            //2. when there is a valid result produced, in this case the error string
+            //  contains actually a warning that is handled above
             setStatus(Cantor::Expression::Done);
         }
-        else if(m_isHelpRequest || m_isHelpRequestAdditional) //help messages are also part of the error output
+        else if (prompt.trimmed() == QLatin1String("MAXIMA>") )
+        {
+            //prompt is "MAXIMA>", i.e. we're switching to the Lisp-mode triggered by to_lisp(). The output in this case is:
+            //   "Type (to-maxima) to restart, ($quit) to quit Maxima.\n<cantor-prompt>\nMAXIMA> </cantor-prompt>\n"
+            //Or we're already in the Lisp mode and just need to show the result of the lisp evaluation.
+            if (static_cast<MaximaSession*>(session())->mode() != MaximaSession::Lisp)
+                static_cast<MaximaSession*>(session())->setMode(MaximaSession::Lisp);
+
+            auto* result = new Cantor::TextResult(errorContent.trimmed());
+            setResult(result);
+            qDebug()<<"setting status to DONE";
+            setStatus(Cantor::Expression::Done);
+        }
+        else if (prompt.trimmed() != QLatin1String("MAXIMA>") && static_cast<MaximaSession*>(session())->mode() == MaximaSession::Lisp)
+        {
+            //"Returning to Maxima:
+            //output:  "Returning to Maxima\n<cantor-result><cantor-text>\n(%o1) true\n</cantor-text></cantor-result>\n<cantor-prompt>(%i2) </cantor-prompt>\n"
+            static_cast<MaximaSession*>(session())->setMode(MaximaSession::Maxima);
+            auto* result = new Cantor::TextResult(errorContent.trimmed());
+            addResult(result);
+            setStatus(Cantor::Expression::Done);
+        }
+        else if(isHelpRequest() || m_isHelpRequestAdditional) //help messages are also part of the error output
         {
             //we've got help result, but maybe additional input is required -> check this
             const int index = prompt.trimmed().indexOf(MaximaSession::MaximaInputPrompt);
@@ -274,7 +323,7 @@ bool MaximaExpression::parseOutput(QString& out)
 
             //set the help result
             errorContent.prepend(QLatin1Char(' '));
-            Cantor::HelpResult* result = new Cantor::HelpResult(errorContent);
+            auto* result = new Cantor::HelpResult(errorContent);
             setResult(result);
 
             //if a new input prompt was found, no further input is expected and we're done
@@ -285,13 +334,16 @@ bool MaximaExpression::parseOutput(QString& out)
         }
         else
         {
-            errorContent = errorContent.replace(QLatin1String("\n\n"), QLatin1String("\n"));
-            setErrorMessage(errorContent);
-            setStatus(Cantor::Expression::Error);
+            if (isInternal())
+                setStatus(Cantor::Expression::Done); //for internal commands no need to handle the error output
+            else
+            {
+                errorContent = errorContent.replace(QLatin1String("\n\n"), QLatin1String("\n"));
+                setErrorMessage(errorContent);
+                setStatus(Cantor::Expression::Error);
+            }
         }
     }
-
-    return true;
 }
 
 void MaximaExpression::parseResult(const QString& resultContent)
@@ -334,16 +386,25 @@ void MaximaExpression::parseResult(const QString& resultContent)
 
     const int latexContentStart = resultContent.indexOf(QLatin1String("<cantor-latex>"));
     //Handle system maxima output for plotting commands
-    if (m_isPlot && textContent.endsWith(QString::fromLatin1("\"%1\"]").arg(m_tempFile->fileName())))
+    if (m_isPlot)
     {
-        m_plotResultIndex = results().size();
-        // Gnuplot could generate plot before we parse text output from maxima and after
-        // If we already have plot result, just add it
-        // Else set info message, and replace it by real result in imageChanged function later
-        if (m_plotResult)
-            result = m_plotResult;
+        // plot/draw command can be part of a multi-line input having other non-plot related commands.
+        // parse the output of every command of the multi-line input and only add plot result if
+        // the output has plot/draw specific keywords
+        if ( (!m_isDraw && textContent.endsWith(QString::fromLatin1("\"%1\"]").arg(m_tempFile->fileName())))
+            || (m_isDraw && (textContent.startsWith(QLatin1String("[gr2d(explicit")) || textContent.startsWith(QLatin1String("[gr3d(explicit"))) ) )
+        {
+            m_plotResultIndex = results().size();
+            // Gnuplot could generate plot before we parse text output from maxima and after
+            // If we already have plot result, just add it
+            // Else set info message, and replace it by real result in imageChanged function later
+            if (m_plotResult)
+                result = m_plotResult;
+            else
+                result = new Cantor::TextResult(i18n("Waiting for the plot result"));
+        }
         else
-            result = new Cantor::TextResult(i18n("Waiting for the plot result"));
+            result = new Cantor::TextResult(textContent);
     }
     else if (latexContentStart != -1)
     {
@@ -365,6 +426,7 @@ void MaximaExpression::parseResult(const QString& resultContent)
                 else if(latexContent[i]==QLatin1Char('}'))
                     pcount--;
 
+
                 if(pcount==0)
                     break;
             }
@@ -419,11 +481,13 @@ void MaximaExpression::imageChanged()
 {
     if(m_tempFile->size()>0)
     {
-#ifdef WITH_EPS
-        m_plotResult = new Cantor::EpsResult( QUrl::fromLocalFile(m_tempFile->fileName()) );
-#else
         m_plotResult = new Cantor::ImageResult( QUrl::fromLocalFile(m_tempFile->fileName()) );
-#endif
+        /*
+        QSizeF size;
+        const QImage& image = Cantor::Renderer::pdfRenderToImage(QUrl::fromLocalFile(m_tempFile->fileName()), 1., false, &size);
+        m_plotResult = new Cantor::ImageResult(image);
+        */
+
         // Check, that we already parse maxima output for this plot, and if not, keep it up to this moment
         // If it's true, replace text info result by real plot and set status as Done
         if (m_plotResultIndex != -1)
